#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
    --server=tiny.my.odoo.com:8069
    --user=chs  (getuser)
    --password=xxx  (prompt)
    --database=tiny_belgium
    
    --from=19/02/2009   (today)
    --to=25/03/2009     (today)
    --max-diff=10
    
    analytic-account   (name_search must return 1 result)
    message


    search timesheet with from_date <= --from AND to_date >= --to
    for each:
        if date not between --from and --to
            continue
""" 

import optparse
import openobject
import openobject.optparseutil
import datetime
import dateutil.relativedelta
import dateutil.rrule
import dateutil.parser

DFMT = '%Y-%m-%d'
DTFMT = '%Y-%m-%d %H:%M:%S'
TODAY = datetime.date.today()

parser = optparse.OptionParser(usage="%prog [options] analytic-account message", version="0.9")

CnxFact = openobject.optparseutil.ConnectionOptionFactory(parser)

parser.add_option(openobject.optparseutil.DateOption('-f', '--from', dest='from_', default=TODAY, type='date'))
parser.add_option(openobject.optparseutil.DateOption('-t', '--to', dest='to', default=TODAY, type='date'))
parser.add_option('--max-diff', dest='max_diff', default=10, type='int')

opt, args = parser.parse_args()

if len(args) != 2:
    parser.error('Invalid arguments')

wanted_analytic_account, message = args

cnx = CnxFact()

analytic_accounts = openobject.xmlrpc.Object(cnx, 'account.analytic.account')

aans = analytic_accounts.name_search(wanted_analytic_account)
if len(aans) == 0:
    parser.exit(1, "Error: Unable to find the analytic account\n")
if len(aans) != 1:
    sep = "\n*/ "
    msg = "Error: More than one analytic have been found:%s%s\n\nPlease affine your search\n" % (sep, sep.join(aa[1] for aa in aans))
    parser.exit(1, msg)

aaid = aans[0][0]

timesheets = openobject.xmlrpc.Object(cnx, 'hr_timesheet_sheet.sheet')
timesheet_lines = openobject.xmlrpc.Object(cnx, 'hr.analytic.timesheet')

optfrom = opt.from_.strftime(DTFMT)
optto = (opt.to + dateutil.relativedelta.relativedelta(days=+1, seconds=-1)).strftime(DTFMT)

#    optfrom between date_from and date_to
# OR optto   -----------------------------
# OR (optfrom < date_from AND optto > date_to)
Q = ['|', '|', '&', ('date_from', '<=', optfrom), ('date_to', '>=', optfrom), 
               '&', ('date_from', '<=', optto), ('date_to', '>=', optto),
               '&', ('date_from', '>', optfrom), ('date_to', '<', optto),
    ]

tsdata = timesheets.select(['&', ('user_id', '=', cnx.userid)] + Q, ['date_from', 'date_to'])

if not tsdata:
    parser.exit(0, "No timesheet found\n")

timesheet_line_data = timesheet_lines.default_get(['product_id', 'product_uom_id', 'user_id', 'general_account_id', 'journal_id'])
timesheet_line_data['name'] = message
timesheet_line_data['account_id'] = aaid

for ts in tsdata:
    print "Processing timesheet %s -> %s" % (ts['date_from'], ts['date_to'])
    dtstart = max([dateutil.parser.parse(ts['date_from']).date(), opt.from_])
    until = min([dateutil.parser.parse(ts['date_to']).date(), opt.to])

    for curday in dateutil.rrule.rrule(dateutil.rrule.DAILY, dtstart=dtstart, until=until):
        timesheets.write([ts['id']], {'date_current': curday.strftime(DFMT)})
        curdiff = timesheets.read([ts['id']], ['total_difference_day'])[0]['total_difference_day']
        if (curdiff * 60) > opt.max_diff:
            print "\t%s: Creating a timesheet line of %d minutes" % (curday.strftime(DFMT), curdiff*60)
            
            timesheet_line_data.update({'date': curday.strftime(DFMT), 'unit_amount': curdiff})
            
            ocua = timesheet_lines.on_change_unit_amount(False, 
                                                         timesheet_line_data['product_id'], 
                                                         timesheet_line_data['unit_amount'], 
                                                         timesheet_line_data['product_uom_id'])
            timesheet_line_data.update(ocua['value'])

            timesheet_lines.create(timesheet_line_data)

