#!/usr/bin/python
# -*- coding: utf-8 -*-

__all__ = ['ConnectionOptionFactory', 'DateOption']

import getpass
import optparse
from xmlrpc import Connection
import dateutil.parser

def _getuser():
    try:
        return getpass.getuser()
    except:
        return None

class ConnectionOptionFactory(object):
    def __init__(self, option_parser, 
                 default_user=_getuser(), 
                 default_server='http://tiny.my.odoo.com:8069',
                 default_database='tiny_belgium',
                 prompt_password=True,
                ):
        self.op = option_parser
        grp = optparse.OptionGroup(self.op, 'Connection Options')

        grp.add_option('-u', '--user', dest='user', default=default_user, help='Username [%default]')

        password_help = 'Password'
        self.prompt_password = prompt_password
        if prompt_password:
            password_help += '. If not provided, it will be prompted.'
        grp.add_option('-p', '--password', dest='password', default=None, help=password_help)
        
        grp.add_option('-s', '--server', dest='server', default=default_server, help='Server [%default]')

        grp.add_option('-d', '--database', dest='database', default=default_database, help='Database [%default]')
        
        self.op.add_option_group(grp)

    def __call__(self):
        opt, _ = self.op.parse_args()
        if not opt.user:
            self.op.error('No user specified')
        
        if not opt.password and self.prompt_password:
            opt.password = getpass.getpass('Password: ')

        if opt.server.startswith('http://'):
            opt.server = opt.server[7:]

        s = opt.server.rsplit(':', 1)
        opt.server = s.pop(0)
        try:
            opt.port = s and int(s[0]) or 8069
        except:
            self.op.error('Invalid server')

        try:
            return Connection(server=opt.server, port=opt.port, dbname=opt.database, login=opt.user, password=opt.password)
        except Exception, e:
            self.op.error(e)


class DateOption(optparse.Option):
    def check_date(option, opt, value):
        try:
            return dateutil.parser.parse(value, dayfirst=True, yearfirst=True).date()
        except ValueError, ve:
            raise optparse.OptionValueError('option %s: invalid date value: %s', (opt, ve))

    TYPES = optparse.Option.TYPES + ('date',)
    TYPE_CHECKER = optparse.Option.TYPE_CHECKER.copy()
    TYPE_CHECKER['date'] = check_date


