import xmlrpclib
from socket import error as socket_error

# TODO abastract the use of the context

class Connection(object):
    def __init__(self, server="localhost", port=8069, dbname="terp", login=None, password=None):
        self.server, self.port = server, port
        self._url = "http://%s:%d/xmlrpc/common" % (server, port)
        self._sock = xmlrpclib.ServerProxy(self._url)
        self.dbname = dbname
        self.login, self.password = login, password
        try:
            self.userid = self._sock.login(dbname, login, password)
        except socket_error, se:
            raise Exception('Unable to connect to http://%s:%d: %s' % (server, port, se.args[1]))
        if not self.userid:
            raise Exception("Unable to connect to database %s using %s" % (dbname, login,))


class Database(object):
    def __init__(self, server="localhost", port=8069, supadminpass='admin'):
        self.supadminpass = supadminpass
        self.server, self.port = server, port
        self._url = "http://%s:%d/xmlrpc/db" % (server, port)
        self._sock = xmlrpclib.ServerProxy(self._url)
        
    def __getattr__(self, name):
        def proxy(*args, **kwargs):
            return getattr(self._sock, name)(self.supadminpass, *args, **kwargs)
        return proxy
    
    def list(self):
        return self._sock.list()

class Object(object):
    def __init__(self, connection, model):
        self._connection = connection
        self._model = model
        self._sock = xmlrpclib.ServerProxy("http://%s:%d/xmlrpc/object" % (connection.server, connection.port))
        
    def __getattr__(self, name):
        def proxy(*args, **kwargs):
            return self._sock.execute(self._connection.dbname, self._connection.userid, self._connection.password, self._model, name, *args, **kwargs)
        return proxy

    def select(self, domain = None, fields=None):
        ids = self.search(domain or [])
        return self.read(ids, fields or [])

class Wizard(object):
    def __init__(self, connection, name):
        self._connection = connection
        self._name = name
        self._sock = xmlrpclib.ServerProxy("http://%s:%d/xmlrpc/wizard" % (connection.server, connection.port))
        self._id = self._sock.create(self._connection.dbname, self._connection.userid, self._connection.password, self._name)

    def __getattr__(self, state):
        def proxy(**kwargs):
            return self._sock.execute(self._connection.dbname, self._connection.userid, self._connection.password, self._id, kwargs, state)
        return proxy

def demo():
    db = Database()
    print repr(db.list())

    cnx = Connection(dbname="lpot", login="admin", password="admin")
    modules = Object(cnx, "ir.module.module")
    
    ids = modules.search([('state', '=', 'installed')])
    for p in modules.read(ids, ['name']):
        print p['name']

if __name__ == '__main__':
    demo()
