<?php

require_once('IXR.lib.php');

class OpenObject_Connection {
    
    function __construct($server='localhost', $port=8069, $dbname='terp', $login=false, $password=false) {
        $this->server = $server;
        $this->port = $port;
        $this->dbname = $dbname;
        $this->login = $login;
        $this->password = $password;
        $client = new IXR_Client($server, 'common', $port);
        $result = $client->query('login', $dbname, $login, $password);
        if (!$result) {
            if ($client->getErrorCode() === -32300) {
                throw new Exception("Unable to connect to http://$server:$port: ".$client->getErrorMessage());
            }
            throw new Exception($client->getErrorMessage());
        }
        $this->userid = $client->getResponse();
        if (!$this->userid) {
            throw new Exception("Unable to connect to database $dbname using $login");
        }
    }
}


class OpenObject_Database {
    private $_client;

    function __construct($server='localhost', $port=8069, $supadminpass='admin') {
        $this->server = $server;
        $this->port = $port;
        $this->supadminpass = $supadminpass;
        $this->_client = new IXR_Client($server, 'db', $port);
    }

    function __call($method, $arguments) {
        $prefix = array($method);
        if ($method !== 'list') {
            $prefix[] = $this->supadminpass;
        }
        $result = call_user_func_array(array($this->_client, 'query'), array_merge($prefix, $arguments));
        if (!$result) {
            throw new Exception($this->_client->getErrorMessage());
        }
        return $this->_client->getResponse();
    }
}


class OpenObject_Object {
    private $_connection;
    private $_model;
    private $_client;
    private $_prefix;

    function __construct($connection, $model) {
        $this->_connection = $connection;
        $this->_model = $model;
        $this->_client = new IXR_Client($connection->server, 'object', $connection->port);
        $this->_prefix = array('execute', $connection->dbname, $connection->userid, $connection->password, $model);
    }

    function __call($method, $arguments) {
        $result = call_user_func_array(array($this->_client, 'query'), array_merge($this->_prefix, array($method), $arguments));
        if (!$result) {
            throw new Exception($this->_client->getErrorMessage());
        }
        return $this->_client->getResponse();
    }
}


class OpenObject_Wizard {
    private $_connection;
    private $_name;
    private $_client;
    private $_prefix;

    function __construct($connection, $model) {
        $this->_connection = $connection;
        $this->_name = $name;
        $this->_client = new IXR_Client($connection->server, 'wizard', $connection->port);

        $result = $this->_client->query('create', $connection->dbname, $connection->userid, $connection->password, $name);
        if (!$result) {
            throw new Exception($this->_client->getErrorMessage());
        }
        $id = $this->_client->getResponse();
        $this->_prefix = array('execute', $connection->dbname, $connection->userid, $connection->password, $id);
    }

    function __call($state, $arguments) {
        $result = call_user_func_array(array($this->_client, 'query'), array_merge($this->_prefix, $arguments, array($state)));
        if (!$result) {
            throw new Exception($this->_client->getErrorMessage());
        }
        return $this->_client->getResponse();
    }
}


function demo() {
    $db = new OpenObject_Database();
    print_r($db->list());
     
    $cnx = new OpenObject_Connection('localhost', 8069, 'test', 'admin', 'a');
    $modules = new OpenObject_Object($cnx, 'ir.module.module');
    $ids = $modules->search(array(array('state', '=', 'installed')));
    foreach($modules->read($ids, array('name')) as $p) {
        echo $p['name'];
        echo "\n";
    }
}

if (realpath($_SERVER['PHP_SELF']) === realpath(__FILE__)) {
    demo();
}

?>
