#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import os
import tempfile
import tarfile
import shutil

def log(msg):
    print msg

def main(tgzfilename, addonspath):

    LNGMAPPING = {
        'ca': 'ca_ES',
        'et': 'et_EE',
    }

    tmpdir = tempfile.mkdtemp()
    log("Extracting %s to %s" % (tgzfilename, tmpdir))
    tar = tarfile.open(tgzfilename)
    tar.extractall(tmpdir)
    tar.close()
    
    for m in os.listdir(tmpdir):
        if not os.path.exists(os.path.join(addonspath, m)):
            log("Destination module %s doen't exists: skip it" % (m,))
            continue

        log("Processing module %s" % (m,))
        lp_m = m.replace('_', '-')
        potfile = os.path.join(tmpdir, m, 'i18n', lp_m + '.pot')
        if os.path.exists(potfile):
            shutil.move(potfile, os.path.join(addonspath, m, 'i18n', m + '.pot'))
        else:
            log("WARNING: the pot file doesn't exists")

        for f in os.listdir(os.path.join(tmpdir, m, 'i18n')):
            lng = f[len(m)+1:-3]
            if '_' not in lng:
                lng = LNGMAPPING.get(lng, '%s_%s' % (lng, lng.upper(),))

            shutil.move(os.path.join(tmpdir, m, 'i18n', f), os.path.join(addonspath, m, 'i18n', lng + '.po'))
    
    shutil.rmtree(tmpdir)
    return 0

if __name__ == '__main__':
    import sys
    if len(sys.argv) != 3:
        print "Usage:"
        print "\t%s <launchpad-export.tar.gz> <addons path>" % (sys.argv[0],)
        exit(1)
        
    exit(main(sys.argv[1], sys.argv[2]))

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

