#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import optparse
import getpass
import openobject.xmlrpc as rpc
from math import modf

def getuser():
    try:
        return getpass.getuser()
    except:
        return None

parser = optparse.OptionParser(usage="%prog [options] <i[n]|s[tate]|o[ut]>", version='1.2')

parser.add_option('-u', '--user', dest='user', default=getuser(), help="Username [%default]")
parser.add_option('-p', '--password', dest='password', default=None, help="Password. If not provided, it will be prompted to the console.")
parser.add_option('-s', '--server', dest='server', default='http://tiny.my.odoo.com:8069', help='Server [%default]')
parser.add_option('-d', '--database', dest='database', default='tiny_belgium', help='Database [%default]')

(opt, args) = parser.parse_args()

if len(args) != 1:
   parser.error('Please provide a way to sign: in or out')

way = args[0].lower()
if way not in ('i', 'in', 's', 'state', 'o', 'out'):
    parser.error('Invalid way')
    
if not opt.user:
    parser.error('No user specifed')

if not opt.password:
    opt.password = getpass.getpass('Password: ')

if opt.server.startswith('http://'):
    opt.server = opt.server[7:]

s = opt.server.rsplit(':', 1)
opt.server = s.pop(0)
try:
    opt.port = s and int(s[0]) or 8069
except:
    parser.error('Invalid server')

try:
    cnx = rpc.Connection(server=opt.server, port=opt.port, dbname=opt.database, login=opt.user, password=opt.password)
except Exception, e:
    parser.error(e)

wiz = rpc.Wizard(cnx, 'hr_timesheet_sheet.current.open')
wr = wiz.init()['action']

ctid = wr.get('res_id')

timesheet = rpc.Object(cnx, wr['res_model'])
if ctid is None:
    ctid = timesheet.create({})

if way[0] != 's':
    action = getattr(timesheet, {'i': 'sign_in', 'o': 'sign_out'}[way[0]])
    action([ctid], {})
    
fields = 'state_attendance', 'total_attendance_day', 'total_attendance'
fg = timesheet.fields_get(fields)
values = timesheet.read(ctid, fields)
for field in fields:
    value = values[field]
    if fg[field]['type'] == 'selection':
        value = dict(fg[field]['selection'])[value]
    elif fg[field]['type'] == 'float':
        s, m = modf(value)
        value = "%.2d:%.2d" % (m, s*60)
    print '%s: %s' % (fg[field]['string'], value)

