#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import openobject.xmlrpc as rpc
from time import sleep
import sys
import os.path
import StringIO
import base64
import tarfile
import atexit
import optparse

parser = optparse.OptionParser(usage="%prog [options] <destination directory>", version='1.1')
parser.add_option('-q', '--quiet', dest='quiet', default=False, action="store_true", help="Disable the output")
parser.add_option('-d', '--database', dest="dbname", default="updtlng", help="Specify the temporary database name. This database must NOT exists")
parser.add_option('-n', '--no-languages', dest='nolang', default=False, action="store_true", help="Do not install the languages. Use this to update only pot files")

(opt, args) = parser.parse_args()
dbname = opt.dbname

if len(args) != 1:
    print >> sys.stderr, "Please provide a destination directory"
    parser.print_usage()
    exit(1)

destdir = args[0]
if not os.path.exists(destdir):
    print >> sys.stderr, "%s does not exist" % destdir
    exit(1)

def log(msg):
    if not opt.quiet:
        print msg

db = rpc.Database()

if dbname in db.list():
    log('Database %s exists' % dbname)
    exit(1)

log('Create database %s' % dbname)
dbid = db.create(dbname, True, 'en_US', 'admin')

while True:
    sleep(0.5)
    p, users = db.get_progress(dbid)
    if p == 1.0:
        break

def out():
    log('Drop database %s' % dbname)
    db.drop(dbname)

atexit.register(out)

log('Connect to database')
cnx = rpc.Connection(dbname=dbname, login="admin", password="admin")

if opt.nolang:
    languages = []
else:
    wiz_lang = rpc.Wizard(cnx, 'module.lang.install')
    res = wiz_lang.init()

    languages = res['fields']['lang']['selection']

    for lngcode, lngname in languages:
        log("Install language %s" % lngname)
        wiz_lang.start(form={'lang': lngcode})

nottoinstall = []   
#nottoinstall = ('report_account','board_account', 'profile_accounting',)

log('Get list of all modules')
modules = rpc.Object(cnx, "ir.module.module")
modids = dict([(m['id'], m['name']) for m in modules.select(['!', ('name', 'in', nottoinstall)], ['id', 'name'])])

log("Mark modules to be installed")
log(" * " + "\n * ".join(modids.values()))
res = modules.button_install(modids.keys())

log("Install modules (this take a while, really)")
wiz = rpc.Wizard(cnx, 'module.upgrade')
wiz.init()
wiz.start()

try:
    crm = rpc.Object(cnx, 'crm.menu.config_wizard')
    fields = crm.fields_get_keys()
except:
    pass
else:
    log("Run the configure wizard of the module 'crm_configuration'")
    vals = dict.fromkeys(fields, True)
    vals['name'] = 'OpenERP'
    crmid = crm.create(vals)
    crm.action_create([crmid])

def export(obj, lngcode, lngname):
    if lngname:
        log("Export language %s" % lngname)
    elid = obj.create({'lang': lngcode, 'format': 'tgz'})
    obj.act_getfile([elid])
    tgz = obj.read([elid], ['data'], {'bin_size':False})[0]['data']

    buf = StringIO.StringIO(base64.decodestring(tgz))
    buf.seek(0)
	
    tar = tarfile.open(mode='r:gz', fileobj=buf)
    for t in tar:
        tar.extract(t, destdir)

    tar.close()
    buf.close()

export_lang = rpc.Object(cnx, 'wizard.module.lang.export')

for lngcode,lngname in languages:
    export(export_lang, lngcode, lngname)

log("Export POT file")
export(export_lang, False, False)

log('zog zog. Job done. You can now compress the po(t) files with the following command')
log('$> find -L addons -name "*.po" -or -name "*.pot" | xargs splittar -z -m 100KB -f translations.tgz')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

